package com.gx.obe.http.json;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.gx.obe.http.jsons.JSONUtil;
import com.gx.obe.http.util.ObjectUtil;

import net.sf.ezmorph.object.DateMorpher;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import net.sf.json.JsonConfig;
import net.sf.json.util.CycleDetectionStrategy;
import net.sf.json.util.JSONUtils;

public class JsonUtil {

	/**
	 * @Title: strToObject @Description: String 转换成 对象 @param: @param
	 * <T> @param: @param str @param: @param clazz @param: @return @return:
	 * T @throws
	 */
	@Deprecated
	public static <T> T strToObject(String str, Class<T> clazz) {
		if (str == null || str.isEmpty()) {
			return null;
		}
		JSONUtils.getMorpherRegistry()
				.registerMorpher(new DateMorpher(new String[] { "yyyy-MM-dd HH:mm:ss", "yyyy-MM-dd" }));
		JsonConfig jsonConfig = new JsonConfig();
		jsonConfig.registerJsonValueProcessor(Date.class, new JsonDateValueProcessor());
		JSONObject jsonObject = JSONObject.fromObject(str, jsonConfig);
		@SuppressWarnings("unchecked")
		T t = (T) JSONObject.toBean(jsonObject, clazz);
		return t;
	}

	/**
	 * @Title: objectToStr @Description: 将对象转换成字符串 @param: @param <T> @param: @param
	 * t @param: @return @return: String @throws
	 */
	public static <T> String objectToStr(T t) {
		if (ObjectUtil.isNull(t)) {
			return null;
		}
		JsonConfig jsonConfig = new JsonConfig();
		jsonConfig.registerJsonValueProcessor(Date.class, new JsonDateValueProcessor());
		jsonConfig.setCycleDetectionStrategy(CycleDetectionStrategy.LENIENT);
		JSONObject jsonObject = JSONObject.fromObject(t, jsonConfig);
		return jsonObject.toString();
	}

	public static <T> String objectToStr(T t, String[] strExcludes) {
		JsonConfig jsonConfig = new JsonConfig();
		jsonConfig.registerJsonValueProcessor(Date.class, new JsonDateValueProcessor());
		jsonConfig.setCycleDetectionStrategy(CycleDetectionStrategy.LENIENT);
		if (null != strExcludes) {
			jsonConfig.setExcludes(strExcludes);
		}
		JSONObject jsonObject = JSONObject.fromObject(t, jsonConfig);
		return jsonObject.toString();
	}

	/**
	 * @Title: strToList @Description: String转换List对象集合 @param: @param
	 * <T> @param: @param str @param: @param clazz @param: @return @return:
	 * List<T> @throws
	 */
	@SuppressWarnings({ "unchecked" })
	public static <T> List<T> strToList(String str, Class<T> clazz) {
		if (str == null || str.isEmpty()) {
			return new ArrayList<>();
		}
		JSONUtils.getMorpherRegistry()
				.registerMorpher(new DateMorpher(new String[] { "yyyy-MM-dd HH:mm:ss", "yyyy-MM-dd" }));
		JsonConfig jsonConfig = new JsonConfig();
		jsonConfig.setCycleDetectionStrategy(CycleDetectionStrategy.LENIENT);
		JSONArray json = JSONArray.fromObject(str, jsonConfig);
		JSONObject object = null;
		T t = null;
		List<T> list = new ArrayList<>();
		for (int i = 0; i < json.size(); i++) {
			object = JSONObject.fromObject(json.get(i));
			t = (T) JSONObject.toBean(object, clazz);
			list.add(t);
		}
		return list;
	}

	/**
	 * @Title: strToObject @Description: 为json的返回值中的对象转换 @param: @param
	 * <T> @param: @param result @param: @param clazz @param: @param
	 * classMap @param: @return @return: T @throws
	 */
	@SuppressWarnings("unchecked")
	public static <T> T strToObject(String result, Class<T> clazz,
			@SuppressWarnings("rawtypes") Map<String, Class> classMap) {
		if (ObjectUtil.isNull(result)) {
			return null;
		}
		JSONUtils.getMorpherRegistry()
				.registerMorpher(new DateMorpherEx(new String[] { "yyyy-MM-dd HH:mm:ss", "yyyy-MM-dd" }));
		JSONObject jsonObj = JSONObject.fromObject(result);
		return (T) JSONObject.toBean(jsonObj, clazz, classMap);
	}
	
	@Deprecated
	@SuppressWarnings("unchecked")
	public static <T> T objectToObject(Object result, Class<T> clazz,
			@SuppressWarnings("rawtypes") Map<String, Class> classMap) {
		if (ObjectUtil.isNull(result)) {
			return null;
		}
		DateMorpherEx data = new DateMorpherEx(new String[] { "yyyy-MM-dd HH:mm:ss", "yyyy-MM-dd" });
		data.setDefaultValue(null);
		JSONUtils.getMorpherRegistry().registerMorpher(data);
		JSONObject jsonObj = JSONObject.fromObject(result);
		return (T) JSONObject.toBean(jsonObj, clazz, classMap);
	}

	/**
	 * @Title: ObjectToObject @Description: 为json的返回值中的对象转换 @param: @param
	 * <T> @param: @param result @param: @param clazz @param: @param
	 * classMap @param: @return @return: T @throws
	 */
	@SuppressWarnings("unchecked")
	public static <T> T ObjectToObject(Object result, Class<T> clazz) {
		if (ObjectUtil.isNull(result)) {
			return null;
		}
		JSONUtils.getMorpherRegistry()
				.registerMorpher(new DateMorpher(new String[] { "yyyy-MM-dd HH:mm:ss", "yyyy-MM-dd" }));
		JSONObject jsonObj = JSONObject.fromObject(result);
		return (T) JSONObject.toBean(jsonObj, clazz);
	}

	@SuppressWarnings("unchecked")
	public static <T> List<T> StrToCollection(String result, Class<T> clazz) {
		if (ObjectUtil.isNull(result)) {
			return null;
		}
		JSONUtils.getMorpherRegistry()
				.registerMorpher(new DateMorpher(new String[] { "yyyy-MM-dd HH:mm:ss", "yyyy-MM-dd" }));
		JSONArray jsonArray = JSONArray.fromObject(result);
		return (List<T>) JSONArray.toCollection(jsonArray, clazz);
	}

	@SuppressWarnings("unchecked")
	public static Map<String, String> strToMap(String result) {
		if (ObjectUtil.isNull(result)) {
			return null;
		}
		JSONObject jasonObject = JSONObject.fromObject(result);
		return (Map<String, String>) jasonObject;
	}

	@SuppressWarnings("unchecked")
	public static Map<String, Object> strToObejectMap(String result) {
		if (ObjectUtil.isNull(result)) {
			return null;
		}
		JSONObject jasonObject = JSONObject.fromObject(result);
		return (Map<String, Object>) jasonObject;
	}

	/**
	 * @Description: 过滤掉不需要的参数
	 * @author mazc
	 * @param list
	 * @param strExcludes 不需要的参数
	 * @return
	 */
	public static String obejctListToStr(List<Object> list, String[] strExcludes) {
		JsonConfig jsonConfig = new JsonConfig();
		jsonConfig.registerJsonValueProcessor(Date.class, new JsonDateValueProcessor());
		if (null != strExcludes) {
			jsonConfig.setExcludes(strExcludes);
		}
		JSONArray listArray = JSONArray.fromObject(list, jsonConfig);
		return listArray.toString();
	}

	public static String obejctListToStr(String obj1, String obj2) {
		JSONObject jsonObj1 = JSONObject.fromObject(obj1);
		JSONObject jsonObj2 = JSONObject.fromObject(obj2);
		JSONObject jsonThree = new JSONObject();
		jsonThree.putAll(jsonObj1);
		jsonThree.putAll(jsonObj2);
		return jsonThree.toString();
	}

	/**
	 * @Description: 将对象转换成字符串
	 * @author mazc
	 * @param <T>
	 * @param t
	 * @return
	 */
	public static <T> String jsonObjectToStr(T t) {
		if (ObjectUtil.isNull(t)) {
			return null;
		}
		return JSONUtil.toJsonPrettyStr(t);
	}

	/**
	 * @Description: json转成对象 例：str = { id：1；name：张三，sex：6} 返回 User user =
	 *               JsonToBean（stri，User。class）；
	 * @author mazc
	 * @param <T>
	 * @param json
	 * @param beanClass
	 * @return
	 */
	public static <T> T jsonToBean(String json, Class<T> beanClass) {
		if (ObjectUtil.isNull(json)) {
			return null;
		}
		return JSONUtil.toBean(json, beanClass);
	}

	public static <T> T jsonToBean(Object json, Class<T> beanClass) {
		if (ObjectUtil.isNull(json)) {
			return null;
		}
		return JSONUtil.toBean(JSONUtil.toJsonPrettyStr(json), beanClass);
	}

//    public static <T> T jsonToBean(Object json, Class<T> beanClass, Map<String, Class> classMap){
//    	T  t = JSONUtil.toBean(JSONUtil.toJsonPrettyStr(json),beanClass);
//    	for (Map.Entry<String, Class> entry : classMap.entrySet()){  
//    			JSONUtil.toBean(JSONUtil.toJsonPrettyStr(json),entry.getValue());
//     	}  
//    	 
//    	return  
//    }

	/**
	 * @Description: 将json 转换成List对象 例如 String jsonArr =
	 *               "[{\"id\":111,\"name\":\"test1\"},{\"id\":112,\"name\":\"test2\"}]";
	 *               返回List<User> userList = JSONUtil.toList(array, User.class);
	 * @author mazc
	 * @param <T>
	 * @param jsonArr
	 * @param elementType
	 * @return
	 */
	public static <T> List<T> jsonToList(String jsonArr, Class<T> elementType) {
		if (ObjectUtil.isNull(jsonArr)) {
			return null;
		}
		com.gx.obe.http.jsons.JSONArray array = JSONUtil.parseArray(jsonArr);
		return JSONUtil.toList(array, elementType);
	}

	public static <T> List<T> jsonToList(Object jsonArr, Class<T> elementType) {
		if (ObjectUtil.isNull(jsonArr)) {
			return null;
		}
		com.gx.obe.http.jsons.JSONArray array = JSONUtil.parseArray(jsonArr);
		return JSONUtil.toList(array, elementType);
	}

	/**
	 * @Description: 将list<map>转成 list对象 例： List<Map<String, String>> mapList = new
	 *               ArrayList<>(); Map<String, String> map1 = new HashMap<>();
	 *               map.put("id", id); map.put("name", name); mapList.add(map1);
	 *               返回List<user> lis = jsonMapToList(mapList,user.class)
	 * @author mazc
	 * @param <T>
	 * @param mapList
	 * @param elementType
	 * @return
	 */
	public static <T> List<T> jsonMapToList(List<Map<String, String>> mapList, Class<T> elementType) {
		com.gx.obe.http.jsons.JSONArray jsonArray = JSONUtil.parseArray(mapList);
		return jsonArray.toList(elementType);
	}

	/**
	 * @Description: json转换成List<map<String ,Object>>
	 * @author mazc
	 * @param strArr
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static List<Map<String, Object>> jsonToListMap(String strArr) {
		JSONArray jsonArray = JSONArray.fromObject(strArr);
		List<Map<String, Object>> mapListJson = (List<Map<String, Object>>) jsonArray;
		return mapListJson;
	}

	public static <T> T jacksonToBean(String json, Class<T> beanClass) {
		if (ObjectUtil.isNull(json)) {
			return null;
		}
		ObjectMapper obj = new ObjectMapper();
		obj.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
		return obj.convertValue(json, beanClass);
	}

	public static <T> T jacksonToBean(Object json, Class<T> beanClass) {
		if (ObjectUtil.isNull(json)) {
			return null;
		}
		ObjectMapper obj = new ObjectMapper();
		obj.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
		return obj.convertValue(json, beanClass);
	}

	public static <T> List<T> jacksonToList(String jsonArr, Class<T> elementType) {
		if (ObjectUtil.isNull(jsonArr)) {
			return null;
		}
		ObjectMapper MAPPER = new ObjectMapper();
		MAPPER.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);

		JavaType jt = MAPPER.getTypeFactory().constructParametricType(ArrayList.class, elementType);
		try {
			return MAPPER.readValue(jsonArr, jt);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	public static <T> Map<String, T> jacksonToMap(String json, Class<T> elementType) {
		if (ObjectUtil.isNull(json)) {
			return null;
		}
		ObjectMapper MAPPER = new ObjectMapper();
		JavaType jvt = MAPPER.getTypeFactory().constructParametricType(HashMap.class, String.class, elementType);
		try {
			return MAPPER.readValue(json, jvt);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	@SuppressWarnings("unchecked")
	public static <T> Map<String, T> jacksonToStringMap(String json) {
		if (ObjectUtil.isNull(json)) {
			return null;
		}
		ObjectMapper MAPPER = new ObjectMapper();
		try {
			return MAPPER.readValue(json, HashMap.class);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * @Description: 将对象转换成字符串
	 * @author mazc
	 * @param <T>
	 * @param t
	 * @return
	 */
	public static <T> String jacksonObjectToStr(T t) {
		if (ObjectUtil.isNull(t)) {
			return null;
		}
		ObjectMapper MAPPER = new ObjectMapper();
		try {
			return MAPPER.writeValueAsString(t);
		} catch (JsonProcessingException e) {
			e.printStackTrace();
			return null;
		}
	}

}