package com.gx.obe.http.util;

public class CharUtil {
	
	public static final char BACKSLASH = '\\';
	
	public static final char DELIM_START = '{';
	
	public static final char SPACE = ' ';
	
	public static final char TAB = '	';
	
	public static final char DOT = '.';
	
	public static final char SLASH = '/';
	
	public static final char CR = '\r';
	
	public static final char LF = '\n';
	
	public static final char UNDERLINE = '_';
	
	public static final char DASHED = '-';
	
	public static final char COMMA = ',';
	
	public static final char DELIM_END = '}';
	
	public static final char BRACKET_START = '[';
	
	public static final char BRACKET_END = ']';
	
	public static final char COLON = ':';
	
	public static final char DOUBLE_QUOTES = '"';
	
	public static final char SINGLE_QUOTE = '\'';
	
	public static final char AMP = '&';
	
	/**
	 * 是否空白符<br>
	 * 空白符包括空格、制表符、全角空格和不间断空格<br>
	 * 
	 * @param c 字符
	 * @return 是否空白符
	 * @see Character#isWhitespace(int)
	 * @see Character#isSpaceChar(int)
	 */
	public static boolean isBlankChar(char c) {
		return isBlankChar((int) c);
	}
	
	/**
	 * 是否空白符<br>
	 * 空白符包括空格、制表符、全角空格和不间断空格<br>
	 * @param c 字符
	 * @return 是否空白符
	 */
	public static boolean isBlankChar(int c) {
		return Character.isWhitespace(c) || Character.isSpaceChar(c) || c == '\ufeff' || c == '\u202a';
	}
	/**
	 * 字符转为字符串<br>
	 * 如果为ASCII字符，使用缓存
	 * 
	 */
	public static String toString(char c) {
		return ASCIIStrCache.toString(c);
	}
	/**
	 * 给定对象对应的类是否为字符类，字符类包括：
	 * @param value 被检查的对象
	 * @return true表示为字符类
	 */
	public static boolean isChar(Object value) {
		return value instanceof Character || value.getClass() == char.class;
	}
	
	
	/**
	 * 是否为Windows或者Linux（Unix）文件分隔符<br>
	 * Windows平台下分隔符为\，Linux（Unix）为/
	 * 
	 * @param c 字符
	 * @return 是否为Windows或者Linux（Unix）文件分隔符
	 */
	public static boolean isFileSeparator(char c) {
		return SLASH == c || BACKSLASH == c;
	}

	/**
	 * 比较两个字符是否相同
	 * 
	 * @param c1 字符1
	 * @param c2 字符2
	 * @param ignoreCase 是否忽略大小写
	 * @return 是否相同
 	 */
	public static boolean equals(char c1, char c2, boolean ignoreCase) {
		if (ignoreCase) {
			return Character.toLowerCase(c1) == Character.toLowerCase(c2);
		}
		return c1 == c2;
	}
	
	/**
	 * 是否为可见ASCII字符，可见字符位于32~126之间
	 *
	 * <pre>
	 *   CharUtil.isAsciiPrintable('a')  = true
	 *   CharUtil.isAsciiPrintable('A')  = true
	 *   CharUtil.isAsciiPrintable('3')  = true
	 *   CharUtil.isAsciiPrintable('-')  = true
	 *   CharUtil.isAsciiPrintable('\n') = false
	 *   CharUtil.isAsciiPrintable('&copy;') = false
	 * </pre>
	 * 
	 * @param ch 被检查的字符处
	 * @return true表示为ASCII可见字符，可见字符位于32~126之间
	 */
	public static boolean isAsciiPrintable(char ch) {
		return ch >= 32 && ch < 127;
	}

	
}
