package com.gx.obe.http.util;


import java.util.Date;

/**  
 * @Title:  DateRange.java   
 * @Package com.gx.obe.http.util   
 * @Description:   日期范围  
 * @author: XiaoMaPro     
 * @date:   2019年4月26日 下午6:59:15   
 * @version V1.0 
 * @Copyright: 2018 - 2019 www.myspro.com Inc. All rights reserved. 
 *  
 */
public class DateRange extends Range<DateTime> {

	/**
	 * 构造，包含开始和结束日期时间
	 * 
	 * @param start 起始日期时间
	 * @param end 结束日期时间
	 * @param unit 步进单位
	 */
	public DateRange(Date start, Date end, final DateField unit) {
		this(start, end, unit, 1);
	}

	/**
	 * 构造，包含开始和结束日期时间
	 * 
	 * @param start 起始日期时间
	 * @param end 结束日期时间
	 * @param unit 步进单位
	 * @param step 步进数
	 */
	public DateRange(Date start, Date end, final DateField unit, final int step) {
		this(start, end, unit, step, true, true);
	}

	/**
	 * 构造
	 * 
	 * @param start 起始日期时间
	 * @param end 结束日期时间
	 * @param unit 步进单位
	 * @param step 步进数
	 * @param isIncludeStart 是否包含开始的时间
	 * @param isIncludeEnd 是否包含结束的时间
	 */
	public DateRange(Date start, Date end, final DateField unit, final int step, boolean isIncludeStart, boolean isIncludeEnd) {
		super(DateUtil.date(start), DateUtil.date(end), new Steper<DateTime>() {

			@Override
			public DateTime step(DateTime current, DateTime end, int index) {
				DateTime dt = current.offsetNew(unit, step);
				if (dt.isAfter(end)) {
					return null;
				}
				return current.offsetNew(unit, step);
			}
		}, isIncludeStart, isIncludeEnd);
	}

}
